/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*
 *            Sohrab Ismail-Beigi         June 10th, 1999
 *   
 *            written originally by Tairan Wang
 * 
 * Control.c -- the control structure for convergence; some
 *              member functions needed for it to work
 *
 */
#include "header.h"

int Control::if_e_converged(real E) 
{
  if ( ( fabs(E - Etot) < E_tolerance )
       && converged ) {
    Etot_old_old = Etot_old;
    Etot_old = Etot;
    Etot = E;
    return 1;
  } 
  if ( ( fabs(E - Etot) < E_tolerance ) 
       && ( fabs(Etot - Etot_old) < E_tolerance ) )
    converged = 1;
  else 
    converged = 0;
  Etot_old_old = Etot_old;
  Etot_old = Etot;
  Etot = E;
  return 0;
}

/* HACK */
int
Control::if_force_diff_converged(Everything &e)
{
  return 1;
}
/*
int
Control::if_force_diff_converged(Everything &e)
{
  Ioninfo &io = e.ioninfo;

  calc_ionic_forces(e);
  e.iondyninfo.print_force(dft_global_log, e);
  
  int sp, i;

  for(sp = 0; sp < io.nspecies; sp++)
    for(i = 0; i < io.species[sp].natoms; i++)
    if(abs((io.species[sp].forces[i]-io.species[sp].old_forces[i])/
      io.species[sp].forces[i]) > e.cntrl.force_diff_tolerance){
	     dft_log("*** Force differences have not yet converged ***\n\n\n"); 
	return 0;
	}
      dft_log("Force differences converged to better than %e\n", e.cntrl.force_diff_tolerance);
  
  return 1;
  }
  */

void
Control::print(int iter) {
  // print out the converged status.
  if (converged == 1) 
    dft_log("Energy converged to %e Hartree after %d iters :\n", 
	    E_tolerance, iter);
  else 
    dft_log("Energy not yet converged to %e Hartree after %d iters : \n", 
	    E_tolerance, iter);
  dft_log("\t%e\n\t%e %e\n\t%e %e\n",
	      Etot,
	      Etot_old, fabs(Etot - Etot_old),
	      Etot_old_old, fabs(Etot_old - Etot_old_old));
}

void Control::reset_converge() {
  // reset the convergence counter;
  // reset  Etot, Etot_old, and Etot_old_old to 0.
  Etot = Etot_old = Etot_old_old = 0.0; 
}
