/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*------------------------  Energies  -----------------------------------*
 *                                                                       *
 * Structure containing various terms in the energy                      *
 *                                                                       *
 *-----------------------------------------------------------------------*/
#ifndef DFT_ENERGIES_H
#define DFT_ENERGIES_H

class Energies
{
public:
  real KE;       /* Kinetic energy */
  real Eloc;     /* Local pseudopotential energy */
  real Enl;      /* Non-local pseudopotential energy */
  real EH;       /* Hartree energy */
  real Exc;      /* Exchange-correlation energy */
  real Ecore;    /* Pseudopotential core energy */
  real Eewald;   /* Ewald energy */
  real Epulay;   /* Pulay correction energy */
  real Etot;     /* Total:  sum of all above */
  real F;        /* The Helmholtz free energy F = Etot - T*S; only
		  * used when we do finite temperature and calculate 
		  * fillings */
  
  // initialize all to 0.
  Energies(){
    KE=EH=Exc=Eewald=Epulay=Etot=F= 0.0;
    Eloc=Ecore=Enl=0.;
  };


  void print(Output *out){
    out->printf("KE     = %25.16le\n",KE);
    out->printf("Eloc   = %25.16le\n",Eloc);
    out->printf("Enl    = %25.16le\n",Enl);
    out->printf("EH     = %25.16le\n",EH);
    out->printf("Exc    = %25.16le\n",Exc);
    out->printf("Ecore  = %25.16le\n",Ecore);
    out->printf("Eewald = %25.16le\n",Eewald);
    out->printf("Epulay = %25.16le\n",Epulay);
    out->printf("Etot   =                        %25.16le\n",Etot);
    
    //ipd: either get this flag somehow from outside
    // or (current solution) print it always (F==Etot for const fillings)
//    if (einfo.calc_fillings == 1)
    out->printf("F      =                        %25.16le\n",F);
    out->flush();

  };
};

#endif // DFT_ENERGIES_H
