/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*---------------------------  Lattice  ---------------------------------*
 *                                                                       *
 * This defines the (periodic) lattice we are working with, and all      *
 * sorts of useful stuff about the lattice is also stored here,          *
 * mostly for convenience.                                               *
 *                                                                       *
 *-----------------------------------------------------------------------*/

#ifndef DFT_LATTICE_H
#define DFT_LATTICE_H

class Lattice
{
public:
  /* lattice vectors (in the columns) & det(latvec) */
  matrix3 latvec;
  real unit_cell_volume;


  /**********************************************************************/
  /*                                                                    */
  /* NOTE ABOUT LATTICE SCALING     (CSG 22/10/2001)                    */
  /*                                                                    */
  /* The `lattscale' variable was removed from here, in order to make   */
  /* it clear that the scaling is already incorporated into the main    */
  /* lattice variable. the scaling is applied when the input file is    */
  /* processed                                                          */
  /*                                                                    */
  /**********************************************************************/
									
  // lattice scaling. the scaling is applied when the lattice is read
  // from the input file, and is not used after that.
  // Ivan thinks this feature should be removed.
  // real lattscale[3];



  // Another name for latvec.  It must be idential to latvec!
  matrix3 R;
  matrix3 invR,RT,invRT,RTR,invRTR;

  // G = recip lat vectors in rows ( G = 2pi*inv(R) )
  // GT is its traspose
  // GGT = G*GT, and inverse
  matrix3 G,GT,GGT,invGGT;

  Lattice()
    {
      latvec.m[0][0] = latvec.m[0][1] = latvec.m[0][2] =
        latvec.m[1][0] = latvec.m[1][1] = latvec.m[1][2] =
        latvec.m[2][0] = latvec.m[2][1] = latvec.m[2][2] = 0.0;
      unit_cell_volume = 0.0;
    }

  // This takes latvec in, scales, the lattice,
  // and fills in everything else
  void setup(Everything &everything);
  void print(Output *log); // Prints latvec, volume, G, GT
};

#endif // DFT_LATTICE_H
