/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*
 * Gabor Csanyi 8/1/2001
 *
 *
 * the rData class holds a simple array of real numbers
 *
 */

#ifndef DFT_REAL_ARRAY_H
#define DFT_REAL_ARRAY_H

class ComplexArray;

class RealArray
{ 
 public:
  
  int ndata; // number of data elements
  const static int datasize = sizeof(real); //size of the data = sizeof(real)
  real *d;   // data array;

  RealArray(int ndata);

  void init(int);

  void operator=(const RealArray &rd);
  inline void operator=(real r);
  RealArray operator+(const RealArray &rd);
  void operator+=(const RealArray &rd);
  RealArray operator-(const RealArray &rd);
  void operator-=(const RealArray &rd);
  void operator*=(real r);
  RealArray operator*(real r);
  ComplexArray operator*(complex c);

  void zero_out(void);
  void negate(void);
  void randomize(void);
};

RealArray operator*(real r,const RealArray &rd);
ComplexArray operator*(complex c,const RealArray &rd);

real abs2(const RealArray &rd);
real dot(const RealArray &rd1,const RealArray &rd2);
void scale_accumulate(real r,const RealArray &rd1, RealArray &rd2);
void scaled_sum(real r1,RealArray &rd1,
		real r2,RealArray &rd2,
		RealArray &rd3);

#endif //DFT_REAL_ARRAY_H
