/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*-----------------------  Speciesinfo  ---------------------------------*
 *                                                                       *
 * Contains information for one ion species                              *
 *                                                                       *
 * This structure, together with Ioninfo (and IonDyninfo if necessary),  *
 * defines the ions and their pseudopotentials                           *
 * (Member functions in setup_ions.c)                                    *
 *                                                                       *
 *-----------------------------------------------------------------------*/

//IPD: Many of These are PW dependent, therefore must be separated!!!

#ifndef DFT_SPECIESINFO_H
#define DFT_SPECIESINFO_H

class Speciesinfo
{
public:
  double Z;               // nuclear charge of the species
  char name[DFT_MSG_LEN]; // identifier
  int natoms;             // number of atoms
  vector3 *atpos;         // positions of atoms
  double mass;            // ionic mass for dynamics
  int *move;              // mask for whether the ion is allowed to move
  vector3 *forces;        // forces on the atoms in lattice coords
  vector3 *old_forces;    // storage area needed by force_diff convergence
  vector3 *ion_vel;
  double ion_damp;
  double ion_fac;
  IonicPotential pot;
  //  int *maps[48];          // for force symmetry calculation

  // member functions:
  Speciesinfo(double=0,char[]="",int=0,vector3* =0,double=0,int* =0,vector3* =0,vector3* =0,vector3* =0,double=0,double=0);
  ~Speciesinfo();
  void setup(Everything &everything);
  void print(const Output *out, Everything &e);
  void print_force(const Output *out, Everything &e) const;  // print force.
};

#endif // DFT_SPECIESINFO_H
