/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*------------------------  Symmetries  ---------------------------------*
 *                                                                       *
 * Structure containing the Symmetry information for the system          *
 *                                                                       *
 *-----------------------------------------------------------------------*/
#ifndef DFT_SYMMETRIES_H
#define DFT_SYMMETRIES_H

class Symmetries
{
private:
  int nsp;
 public:
  
  // 0 = no symmetries
  // 1 = calculate them automatically
  // 2 = manual symmetries are input (3x3 matrices)
  int calc_symmetries_flag;

  // number of symmetries
  int nrot;

  // symmetries.   Nikolaj Moll  Apr 3 1999
  // sym[] contains the symmetry matrices in relative real-space coordinates.
  // n_sym[] is the corresponding symmetry matrices for charge density mesh,
  //         i.e. the rescaling of mesh-density is included. (ipd -or NOT?)
  // done[0:n.n-1] is a temporary array used for symmetry calculation
  matrix3 sym[48], n_sym[48], f_sym[48];

  // Center for the point symmetries in lattice coordinates (ipd)
  // NB: The planewave part moves the atoms so that Rsymm is zero
  //     For the wavelet part this is unacceptable, because the whole
  //     gridstructure is rigidly conencted with the atomic positions
  // TODO: The planewaves should depart from this model and use Rsym
  //       Also if Rsymm is in absolute coordinates tere may be a way to
  //       change only the origin of the top level
  vector3 Rsym;
  
  // Contains mapping informaton for atoms under symmetries for each
  // species:  the order is map[species][irot][nat], which says which atom
  // of species atom nat gets mapped to under symmetry irot.
  int ***maps;

  // This is a vector as long as the grid in real space which flags
  // whether that point was already symmetrized. Used when symmetrizing
  // the electron density.
  int *done;

  Symmetries();
  ~Symmetries();
  void setup(Everything &e, const Basis &basis);
};

#endif // DFT_SYMMETRIES_H
