/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*-------------------------  calcener.c  --------------------------------*
 *                                                                       *
 * Calculate all the various energy terms in various combinations        *
 * All these energy routines simply take what is given to them in        *
 * as inputs and calculate the corresponding energies; i.e. none of      *
 * of the contents of the inputs are changed.  For example the local     *
 * pseudopot. energy does NOT calculate the local pseudopotential and    *
 * the Hartree energy does NOT solve the poisson equation.               *
 *                                                                       *
 *-----------------------------------------------------------------------*/
#ifndef DFT_CALCENER_H
#define DFT_CALCENER_H

void calc_KE(Everything &everything);
void calc_EH(Everything &everything);
void calc_Exc(Everything &everything);
void calc_Eewald(Everything &everything);
void calc_Epulay(Everything &everything);
void calc_Eloc(Everything &everything);
void calc_Enl(Everything &everything);
void calc_Ecore(Everything &everything);
void calc_Etot(Energies &ener); /* adds up all the energies into ener->Etot */
void calc_F(Everything &everything); /* Helmholtz free energy */
void calc_elec_dependent_energies(Everything &everything);
void calc_core_ewald_pulay_energies(Everything &everything);
void calc_all_energies(Everything &everything);

#endif // DFT_CALCENER_H
