/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

//Only Complex PlaneWaves for now
#ifndef DFT_CHOOSER_H
#define DFT_CHOOSER_H


typedef ComplexMatrix Matrix;

typedef ComplexArray RealSpaceWavefunctionData;
typedef ComplexArray RealSpaceScalarFieldData;
typedef ComplexArray CoeffSpaceWavefunctionData;
typedef ComplexArray CoeffSpaceScalarFieldData;

#if defined PLANEWAVES
// Planewaves
typedef PW_Basis Basis; 
typedef PW_BasisSpec BasisSpec; 

#define WFS_COMPLEX
typedef PW_ComplexColumn GenericColumn;
typedef PW_ComplexColumn RealSpaceWavefunctionColumn;
typedef PW_ComplexColumn RealSpaceScalarFieldColumn;
typedef PW_ComplexColumn CoeffSpaceWavefunctionColumn;
typedef PW_ComplexColumn CoeffSpaceScalarFieldColumn;

typedef PW_IonicPotential IonicPotential;

#elif defined WAVELETS
// Wavelets
typedef WL_Basis Basis;
typedef WL_BasisSpec BasisSpec;

#define WFS_COMPLEX
typedef WL_ComplexColumn GenericColumn;
typedef WL_ComplexColumn RealSpaceWavefunctionColumn;
typedef WL_ComplexColumn RealSpaceScalarFieldColumn;
typedef WL_ComplexColumn CoeffSpaceWavefunctionColumn;
typedef WL_ComplexColumn CoeffSpaceScalarFieldColumn;

typedef WL_IonicPotential IonicPotential;

#else
#error Gotta use some basis, man!!!
#endif


#endif //DFT_CHOOSER_H
