/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int wl_embedding_dep[] = {GRIDSPEC};

extern char processing_error[DFT_LINE_LEN];

int process_wl_embedding(char *input_line,Everything &e)
{
  char embedding_name[DFT_FILENAME_LEN];
  // Make sure we're given a filename.
  if(sscanf(input_line,"%*s %s",embedding_name) != 1)
    {
      sprintf(processing_error,
	      "embedding must be followed by a name.  Input was:\n%s\n",
	      input_line);
      return 1;
    }
  
  if(!strcmp(embedding_name,"sparse"))
    e.basis_spec.use_dense=FALSE;
  else if (!strcmp(embedding_name,"dense"))
    e.basis_spec.use_dense=TRUE;
  else
    die("process_wl_embedding: unknownd embedding %s\n", embedding_name);


  return 0;
}

void
wl_embedding_set_default(Everything &e)
{
  // The default is don't use dense
  e.basis_spec.use_dense=FALSE;
}

void
wl_embedding_print_status(Everything &e, Output *log)
{
  log->printf("scalar-field-embedding dense?: %d\n",e.basis_spec.use_dense);
}

command *
setup_wl_embedding(command *next)
{
  command *cmnd = alloc_command();

  cmnd->number = WL_EMBEDDING;
  cmnd->name = "scalar-field-embedding";
  cmnd->format = "scalar-field-embedding <sparce | dense>";
  cmnd->comments = "# Specifies the embedding of the scalar fields.\n";
  cmnd->dependencies = wl_embedding_dep;
  cmnd->process_command = process_wl_embedding;
  cmnd->print_status = wl_embedding_print_status;
  cmnd->allow_multiple = 0;
  cmnd->has_default = 1;
  cmnd->set_default = wl_embedding_set_default;
  cmnd->next_command = next;

  return cmnd;
}
