/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int band_structure_dep[] = {//-IONIC_DYNAMICS, // this is not supported now
                            -RELAX_IONS,
                            -CALCULATE_FORCES,
                            -ELECTRONIC_MINIMIZATION,
                            CNTRL_SUBSPACE_ROTATION,
                            // must come up with mechanism to select read density
                            // or read potential - i'll do it for the potential now
                            //CNTRL_READ_DENSITY,
                            // until then disable the old code and add the next 2 lines
                            -CNTRL_READ_VLOC, // no need to read the ionic (pseudo)pot
                            CNTRL_READ_VSCLOC, // read the self consistent potential
                            LATTICE,
                            ION,ION_SPECIES,
                            ELEC_CUTOFF,
                            KPOINT,0};

extern char processing_error[DFT_LINE_LEN];

int
process_band_structure(char *input_line,Everything &e)
{
  if (e.elecinfo.subspace_rotation == TRUE)
    {
      sprintf(processing_error,
	      "band-structure can not be used with subspace rotations turned\non (by use of the cntrl-subspace-rotation command).\n\n");
      return 1;
    }
  e.cntrl.band_structure_flag = TRUE;
  return 0;
}

void
band_structure_print_status(Everything &e,Output *log)
{
  if (e.cntrl.band_structure_flag == TRUE)
    log->printf("band-structure\n");
}

command *
setup_band_structure(command *next)
{
  command *cmnd = alloc_command();

  cmnd->number = BAND_STRUCTURE;
  cmnd->name = "band-structure";
  cmnd->format = "band-structure";
  cmnd->comments = "# Perform band structure calculation";
  cmnd->is_overall_action = 1;
  cmnd->process_command = process_band_structure;
  cmnd->print_status = band_structure_print_status;
  cmnd->dependencies = band_structure_dep;
  cmnd->next_command = next;

  return cmnd;
}
