/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int coords_type_dep[] = {0};

char coords_type_names[2][DFT_MSG_LEN] = { "lattice", "cartesian" };
CoordsType coords_type_numbers[2] = { LATTICE_COORDS, CARTESIAN_COORDS };

extern char processing_error[DFT_LINE_LEN];

int process_coords_type(char *input_line,Everything &e)
{
  char key[DFT_MSG_LEN];
  if (sscanf(input_line,"%*s %s",key)!=1)
    {
      sprintf(processing_error,"coords_type must have an argument!\n");
      return 1;
    }
  for (int i=0; i < 2; i++)
    if (MATCH(key,coords_type_names[i]))
      {
	e.ioninfo.coords_type = coords_type_numbers[i];
	return 0;
      }
  sprintf(processing_error,"%s is an unknown coords_type.\n",key);
  return 1;
}

void coords_type_set_default(Everything &e)
{
  // Default is lattice coords
  e.ioninfo.coords_type = LATTICE_COORDS;
}

void coords_type_print_status(Everything &e,Output *log)
{
  for (int i=0; i < 2; i++)
    if (e.ioninfo.coords_type == coords_type_numbers[i])
      log->printf("coords-type %s\n",coords_type_names[i]);
}

command* setup_coords_type(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = COORDS_TYPE;
  cmnd->name = "coords-type";
  cmnd->format = "coords-type lattice | cartesian";
  cmnd->comments = "# Coordinate system to use";
  cmnd->dependencies = coords_type_dep;
  cmnd->has_default = 1;
  cmnd->process_command = process_coords_type;
  cmnd->set_default = coords_type_set_default;
  cmnd->print_status = coords_type_print_status;
  cmnd->next_command = next;

  return cmnd;
}
