/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int elec_fermi_fillings_dep[] = {0};

extern char processing_error[DFT_LINE_LEN];

int process_elec_fermi_fillings(char *input_line,Everything &e)
{
  e.elecinfo.calc_fillings = TRUE;
  if (sscanf(input_line,"%*s %d %lg",
	     &e.elecinfo.niter_recalc_fillings,&e.elecinfo.kT)!=2)
    {
      sprintf(processing_error,
	      "elec-fermi-fillings must be followed by two numbers.\n");
      sprintf(processing_error,
	      "%sInput line was:\n%s\n",processing_error,input_line);
      return 1;
    }
  return 0;
}

void elec_fermi_fillings_set_default(Everything &e)
{
  // Do not calculate by default
  e.elecinfo.calc_fillings = FALSE;
}

void elec_fermi_fillings_print_status(Everything &e,Output *log)
{
  if (e.elecinfo.calc_fillings == TRUE)
    log->printf("elec-fermi-fillings %d %lg\n",
		e.elecinfo.niter_recalc_fillings,e.elecinfo.kT);
}

command* setup_elec_fermi_fillings(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = ELEC_FERMI_FILLINGS;
  cmnd->name = "elec-fermi-fillings";
  cmnd->format = "elec-fermi-fillings <niter> <kT>";
  cmnd->comments = "# Fermi-Dirac fillings will be recomputed every niter electronic iterations\n# at temperature kT (in Hartree)";
  cmnd->dependencies = elec_fermi_fillings_dep;
  cmnd->has_default = 1;
  cmnd->process_command = process_elec_fermi_fillings;
  cmnd->set_default = elec_fermi_fillings_set_default;
  cmnd->print_status = elec_fermi_fillings_print_status;
  cmnd->next_command = next;

  return cmnd;
}
