/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int elec_n_bands_dep[] = {0};

extern char processing_error[DFT_LINE_LEN];

int process_elec_n_bands(char *input_line,Everything &e)
{
  char key[DFT_MSG_LEN];
  sscanf(input_line,"%*s %s",key);
  if (MATCH(key,"automatic"))
    {
      e.cntrl.auto_n_bands = TRUE;
      return 0;
    }
  else if (sscanf(input_line,"%*s %d",&e.elecinfo.nbands)!=1)
    {
      sprintf(processing_error,
	      "elec-n-bands must be followed by either automatic or a number.  Input line was:\n%s\n",input_line);
      return 1;
    }
  else if (e.elecinfo.nbands<=0)
    {
      sprintf(processing_error,
	      "elec-n-bands needs a positive integer as the number of bands.  Input line was:\n%s\n",input_line);
      return 1;
    }
  e.cntrl.auto_n_bands = FALSE;
  return 0;
}

void elec_n_bands_set_default(Everything &e)
{
  // By default, compute number of bands automaticaly as for
  // an insulator (nbands=nelectrons/2)
  e.cntrl.auto_n_bands = TRUE;
}

void elec_n_bands_print_status(Everything &e,Output *log)
{
  if (e.cntrl.auto_n_bands == TRUE)
    log->printf("elec-n-bands automatic\n");
  else
    log->printf("elec-n-bands %d\n",e.elecinfo.nbands);
}

command* setup_elec_n_bands(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = ELEC_N_BANDS;
  cmnd->name = "elec-n-bands";
  cmnd->format = "elec-n-bands automatic | <n>";
  cmnd->comments = "# Number of (occupied) bands for the calculation\n# automatic assumes an insulator and balances ionic charges (Z)\n# A positive integer <n> specifies the number of bands explicitely";
  cmnd->dependencies = elec_n_bands_dep;
  cmnd->process_command = process_elec_n_bands;
  cmnd->has_default = 1;
  cmnd->set_default = elec_n_bands_set_default;
  cmnd->print_status = elec_n_bands_print_status;
  cmnd->next_command = next;

  return cmnd;
}
