/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int ion_relax_param_dep[] = {-ION_DYNAMICS_PARAM,0};

extern char processing_error[DFT_LINE_LEN];

int process_ion_relax_param(char *input_line,Everything &e)
{
  char key[DFT_MSG_LEN];
  if (sscanf(input_line,"%*s %s",key)!=1)
    {
      sprintf(processing_error,"ion-relax-param must have arguments\n");
      return 1;
    }
  for (int sp=0; sp < e.ioninfo.nspecies; sp++)
    if (MATCH(key,e.ioninfo.species[sp].name))
      if (sscanf(input_line,"%*s %*s %lg %lg",
		 &e.ioninfo.species[sp].ion_damp,
		 &e.ioninfo.species[sp].ion_fac)!=2)
	{
	  sprintf(processing_error,
		  "ion-relax-param needs damp and factor.  Input was:\n%s\n",
		  input_line);
	  return 1;
	}
      else
	return 0;
	
  sprintf(processing_error,"Species %s not defined.\n", input_line);
  return 1;
}

void ion_relax_param_print_status(Everything &e,Output *log)
{
  for (int sp=0; sp < e.ioninfo.nspecies; sp++)
    log->printf("ion-relax-param %s %lg %lg\n",
		e.ioninfo.species[sp].name,
		e.ioninfo.species[sp].ion_damp,
		e.ioninfo.species[sp].ion_fac);
}

command* setup_ion_relax_param(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = ION_RELAX_PARAM;
  cmnd->name = "ion-relax-param";
  cmnd->format = "ion-relax-param <species id> <damp> <factor>";
  cmnd->comments = "# Ion relaxation parameters for a species.  Can be specied multiple times";
  cmnd->dependencies = ion_relax_param_dep;
  cmnd->allow_multiple = 1;
  cmnd->process_command = process_ion_relax_param;
  cmnd->print_status = ion_relax_param_print_status;
  cmnd->next_command = next;

  return cmnd;
}
