/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int kpoint_dep[] = {0};

extern char processing_error[DFT_LINE_LEN];

int process_kpoint(char *input_line,Everything &e)
{
  real kx,ky,kz,w;
  if (sscanf(input_line,"%*s %lg %lg %lg %lg",&kx,&ky,&kz,&w)!=4)
    {
      sprintf(processing_error,
	      "kpoint requires 4 arguments.  Input was:\n%s\n",input_line);
      return 1;
    }
  int &n =  e.elecinfo.nstates;
  n++;
  if (n==1) {
      e.elecinfo.input_kvec = (vector3 *)mymalloc(sizeof(vector3)*n,
						  "input_kvec",
						  "process_kpoint()");
      e.elecinfo.input_weight = (real *)mymalloc(sizeof(real)*n,
						 "intpu_weight",
						 "process_kpoint()");
  }
  else {
    e.elecinfo.input_kvec = (vector3 * )myrealloc(e.elecinfo.input_kvec,
						    sizeof(vector3)*n,
						  "input_kvec",
						  "process_kpoint()");
    e.elecinfo.input_weight = (real *)myrealloc(e.elecinfo.input_weight,
						sizeof(real)*n,
						"input_weight",
						"process_kpoint()");
    }
  e.elecinfo.input_kvec[n-1].v[0] = kx;
  e.elecinfo.input_kvec[n-1].v[1] = ky;
  e.elecinfo.input_kvec[n-1].v[2] = kz;
  e.elecinfo.input_weight[n-1]= w;
  return 0;
}

void kpoint_print_status(Everything &e,Output *log)
{
  for (int i=0; i < e.elecinfo.nstates; i++)
    log->printf("kpoint %16.12lf %16.12lf %16.12lf %17.14lf\n",
		e.elecinfo.input_kvec[i].v[0],
		e.elecinfo.input_kvec[i].v[1],
		e.elecinfo.input_kvec[i].v[2],
		e.elecinfo.input_weight[i]);
}

command* setup_kpoint(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = KPOINT;
  cmnd->name = "kpoint";
  cmnd->format = "kpoint <k1> <k2> <k3> <weight>";
  cmnd->comments = "# Specify kpoint at k1 k2 k3 with weight";
  cmnd->dependencies = kpoint_dep;
  cmnd->allow_multiple = 1;
  cmnd->process_command = process_kpoint;
  cmnd->print_status = kpoint_print_status;
  cmnd->next_command = next;

  return cmnd;
}
