/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int latt_scale_dep[] = {LATTICE,0};

extern char processing_error[DFT_LINE_LEN];

int process_latt_scale(char *input_line,Everything &e)
{
  if (sscanf(input_line,"%*s %lg %lg %lg",
	     &e.cntrl.lattscale[0],
	     &e.cntrl.lattscale[1],
	     &e.cntrl.lattscale[2])!=3)
    {
      sprintf(processing_error,
	      "latt-scale requires 3 numeric arguments.  Input was:\n%s\n",
	      input_line);
      return 1;
    }
  int i,j;
  for (i=0; i < 3; i++)
    for ( j=0; j < 3; j++)
      e.lattice.latvec.m[j][i] *= e.cntrl.lattscale[i];

  return 0;
}

void latt_scale_set_default(Everything &e)
{
  // No scaling by default
  for (int i=0; i < 3; i++)
    e.cntrl.lattscale[i] = 1.0;
}

void latt_scale_print_status(Everything &e,Output *log)
{
  log->printf("latt-scale ");
  for (int i=0; i < 3; i++)
    log->printf("%lg ",e.cntrl.lattscale[i]);
  log->printf("\n");
}

command* setup_latt_scale(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = LATT_SCALE;
  cmnd->name = "latt-scale";
  cmnd->format = "latt-scale <s1> <s2> <s3>";
  cmnd->comments = "# Scale lattice vectors by factors s1, s2, s3";
  cmnd->dependencies = latt_scale_dep;
  cmnd->has_default = 1;
  cmnd->process_command = process_latt_scale;
  cmnd->set_default = latt_scale_set_default;
  cmnd->print_status = latt_scale_print_status;
  cmnd->next_command = next;

  return cmnd;
}
