/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/
/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int lattice_dep[] = {0};

extern char processing_error[DFT_LINE_LEN];

int process_lattice(char *input_line,Everything &e)
{
  matrix3 &latvec = e.lattice.latvec;

  if (sscanf(input_line,"%*s %lg %lg %lg %lg %lg %lg %lg %lg %lg",
	     &(latvec.m[0][0]), &(latvec.m[0][1]), &(latvec.m[0][2]), 
	     &(latvec.m[1][0]), &(latvec.m[1][1]), &(latvec.m[1][2]), 
	     &(latvec.m[2][0]), &(latvec.m[2][1]), &(latvec.m[2][2]) )!=9)
    {
      sprintf(processing_error,
	      "lattice requires 9 number arguments.  Input was:\n%s\n",
	      input_line);
      return 1;
    }
  return 0;
}

void lattice_print_status(Everything &e,Output *log)
{
  log->printf("lattice\t\\\n");
  for (int i=0; i < 3; i++)
    {
      log->printf("\t");
      for (int j=0; j < 3; j++)
	log->printf("%20.15le ",e.lattice.latvec.m[i][j]);
      if (i<2)
	log->printf(" \\");
      log->printf("\n");
    }
}

command* setup_lattice(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = LATTICE;
  cmnd->name = "lattice";
  cmnd->format = "lattice <a1> <b1> <c1> <a2> <b2> <c2> <a3> <b3> <c3>";
  cmnd->comments = "# Specifies lattice vectors a, b, c (column vectors)";
  cmnd->dependencies = lattice_dep;
  cmnd->process_command = process_lattice;
  cmnd->print_status = lattice_print_status;
  cmnd->next_command = next;

  return cmnd;
}
