/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

#include "command.h"

int symmetry_matrix_dep[] = {SYMMETRIES,0};

extern char processing_error[DFT_LINE_LEN];

int process_symmetry_matrix(char *input_line,Everything &e)
{
  // If symmetires are not in manual mode, then we have a problem!
  if (e.symm.calc_symmetries_flag != 2) {
    sprintf(processing_error,
            "symmetry-matrix needs symmetries to be called with \"manual\"\nInput line:%s\n", input_line);
    return 1;
  }
  // Read symmetry matrix
  else if (sscanf(input_line,"%*s %lg %lg %lg %lg %lg %lg %lg %lg %lg",
	     &(e.symm.sym[e.symm.nrot].m[0][0]), 
	     &(e.symm.sym[e.symm.nrot].m[0][1]), 
	     &(e.symm.sym[e.symm.nrot].m[0][2]), 
	     &(e.symm.sym[e.symm.nrot].m[1][0]), 
	     &(e.symm.sym[e.symm.nrot].m[1][1]), 
	     &(e.symm.sym[e.symm.nrot].m[1][2]), 
	     &(e.symm.sym[e.symm.nrot].m[2][0]), 
	     &(e.symm.sym[e.symm.nrot].m[2][1]), 
	     &(e.symm.sym[e.symm.nrot].m[2][2]) )!=9)
    {
      sprintf(processing_error,
	      "symmetry-matrix requires 9 number arguments.  Input was:\n%s\n",
	      input_line);
      return 1;
    }
  e.symm.nrot++;
  return 0;
}

void symmetry_matrix_print_status(Everything &e,Output *log)
{
  log->printf("# %d symmetry matrices follow.\n",e.symm.nrot);
  for (int i=0; i < e.symm.nrot; i++)
    {
      log->printf("symmetry-matrix \\\n");
      for (int j=0; j < 3; j++)
	{
	  log->printf("\t");
	  for (int k=0; k < 3; k++)
	    log->printf("%2g ",e.symm.sym[i].m[j][k]);
	  if (j!=2)
	    log->printf("\t\\");
	  log->printf("\n");
	}
    }
}

command* setup_symmetry_matrix(command *next)
{
  command* cmnd = alloc_command();

  cmnd->number = SYMMETRY_MATRIX;
  cmnd->name = "symmetry-matrix";
  cmnd->format = "symmetry-matrix <s00> <s01> <s02> <s10> <s11> <s12> <s20> <s21> <s22>";
  cmnd->comments = "# Specify symmetry operator matrix explicitely.\n# Requires symmetries command to be called with manual argument";
  cmnd->dependencies = symmetry_matrix_dep;
  cmnd->allow_multiple = 1;
  cmnd->process_command = process_symmetry_matrix;
  cmnd->print_status = symmetry_matrix_print_status;
  cmnd->next_command = next;

  return cmnd;
}
