#
# What we will be doing:  minimizing the electronic
# degrees of freedom so as to reach self-consistency
#
electronic-minimization

# Specify the gridspec file.
gridspec gridspec.dft

#
# The lattice vectors for our system:
# the units are Bohr radii, and the lattice vectors
# are the columns of the matrix specified.
# This lattice corresponds to a=5.59 Angstroms.
#
lattice 6.0                0               0                     \
         0                6.0                0                     \
         0                 0                6.0               

#
# This defines the ionic species "Silicon".
# It has an ionic charge of Z=4.0 (4 valence electrons),
# a mass of 28.0855 amus (not used unless you're doing
# ionic dynamics), and its pseudopotential is specified
# in the file "Si/si.pot".  The "none" says there
# are no pulay corrections.
#
ion-species Magnesium 12.0 24.305  

#
# These lines say where are atoms are located.
# The coordinates are in lattice units (i.e. along the
# lattice vectors).  The "1" flag at the end says
# the ions are movable.
#
ion Magnesium  0.500000  0.500000  0.500000  1

spintype z-spin

#
# We'll use the LDA exchange-correlation functional (Perdew-Zunger)
#
elec-ex-corr lsd-teter

#
# The k-points:  k1, k2, k3, followed by the weight
# for the k-point (weights should all up to 1.0).
# k1, k2, and k3 in reciprocal lattice units.
# Here we have 32 k-points, which by time-reversal
# symmetry really correspond to 64 k-points, making
# a 4x4x4 cubic mesh.
#
kpoint 0. 0. 0. 1.0

#
# We will start with random initial wave functions
#
wavefunction read C

#
# Let's do 50 iterations of minimization steps
#
cntrl-max-elec-steps 50

