/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/* 
 * Gabor Csanyi, June 1999
 * 
 * header file for
 *
 * generalized I/O functions to handle serial and MPI in a unified way
 * 
 * dft_fopen()
 * dft_fclose()
 * dft_fread() BUG: assumes data is type double
 * dft_fwrite()
 *
 */

#ifndef DFT_DFT_IO_H
#define DFT_DFT_IO_H

/* 
 * dft_fopen() opens a file and returns the file pointer. arguments
 * are as in standard fopen(). If called in a serial context, it
 * simply executes a system fopen(). If called in an MPI context, it
 * executes a fopen() on the I/O node, and returns a NULL on all the
 * other nodes.
 */
FILE * dft_fopen(char *filename, char *access_string);

/* 
 * dft_fclose() closes the stream pointed to by its argument. if the
 * argument is NULL, it does nothing.
 */
void dft_fclose(FILE *stream);

/*
 * dft_fwrite() called in a scalar context just does an fwrite(). in an
 * MPI context, it does an fwrite if executed in the I/O node, and nothing
 * on any other node. returns the number of bytes written.
 */
size_t dft_fwrite(const void *data, size_t size, size_t nmemb, FILE *stream);
size_t dft_fread(void *data, size_t size, size_t nmemb, FILE *stream);

/* Does a file flush */
int dft_fflush(FILE *stream);

/* Does vfprintf */
int dft_vfprintf(FILE *stream, char *format, va_list ap);

#endif // DFT_DFT_IO_H
