/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*--------------------------- dft_text_FILE.h ---------------------------*
 *                                                                       *
 *  is used in  Professor Arias's file IO suite. (dft_text_FILE.c)       *
 *                                                                       *
 *-----------------------------------------------------------------------*/
#ifndef DFT_TEXT_FILE_H
#define DFT_TEXT_FILE_H

//
// Define dft_text_FILE class:
// It is used to read in text files for setup, etc. that need
// to be distributed to all nodes/processes if we are running
// in a parralelized MPI mode.  In such a case, the main process/node
// reads the entire file as a big string and broadcasts it to everyone.
// Then the processes call routines like dft_text_fscanf or _fgets that
// just go through the string.
//
class dft_text_FILE
{
public:
  int my_id, io_id;
  long int file_length;
  char* file_string;
  char *current_pointer, *old_pointer;
  FILE* file_handle;
  dft_text_FILE();
  ~dft_text_FILE();
  void init();
  void expand_include();
};

/* And functions that use it */
dft_text_FILE *dft_text_fopen(const char *filename, const char *type);
int dft_text_fclose(dft_text_FILE* file);
char * dft_text_fgets(char * s, int n, dft_text_FILE * file);
char * dft_text_fgets(char * s, int n, char c, dft_text_FILE * file);
int dft_text_fscanf( dft_text_FILE *file, const char *fmtin, ...);
void dft_text_rewind(dft_text_FILE* file);
// this subroutine is a hack..!?!?!
void dft_text_fputc_old(char c, dft_text_FILE * file);

#endif // DFT_TEXT_FILE_H
