/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*------------------------  diag_matrix  --------------------------------*
 *                                                                       *
 * diag_matrix class:                                                    *
 *                 a diagonal matrix of scalar elements (diag_matrix.c)  *
 *                                                                       *
 *-----------------------------------------------------------------------*/
#ifndef DFT_DIAG_MATRIX_H
#define DFT_DIAG_MATRIX_H

class diag_matrix
{
  /* data */
public:
  int n;
  scalar *c;

  /* Constructors and destructor */
  diag_matrix();
  diag_matrix(int len);
  diag_matrix(const diag_matrix &d1);
  ~diag_matrix();
  
  /* Operators */
  void operator=(const diag_matrix &d1); /* Non-std: = returns void */
  inline scalar operator()(int i) const {return c[i];}; //i'th element
    
  /* Member functions: */
  /* Does what the constructor does */
  void init(int len);
  /* free up memory */
  void freemem(void) { free(c); }          /* free the memory */
  /* print the matrix to output */
  void print(Output *out);
  /* print the real part */
  void print_real(Output *out);
  /* Zero out the elements of the diag_matrix */
  void zero_out(void);


  //algebra
  void operator*=(const complex &c);
  friend diag_matrix operator*(const complex &, const diag_matrix &);

};

scalar trace(const diag_matrix &);
#endif // DFT_DIAG_MATRIX_H
