/*
    DFT++ is a density functional package developed by the research group
    of Professor Tomas Arias

    Copyright 1996-2003 Sohrab Ismail-Beigi

    This file is part of DFT++.

    DFT++ is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    DFT++ is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with DFT++; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Please see the file CREDITS for a list of authors.

    For academic users, we request that publications using results obtained with
    this software reference

    "New algebraic formulation of density functional calculation," by Sohrab Ismail-Beigi
    and T.A. Arias, Computer Physics Communications 128:1-2, 1-45 (June 2000).

    and, if using the wavelet basis, further reference

    "Multiresolution analysis of electronic structure: semicardinal and wavelet bases,"
    T.A. Arias, Reviews of Modern Physics 71:1, 267-311 (January 1999).

    and 

    "Robust ab initio calculation of condensed matter: transparent convergence through
    semicardinal multiresolution analysis,'' I.P. Daykov, T.A. Arias, and
    Torkel D. Engeness, Physical Review Letters, 90:21, 216402 (May 2003).

    For your convenience, preprints of the above articles may be obtained from
    http://arXiv.org/abs/cond-mat/9909130, 9805262, and 0204411, respectively.
*/

/*--------------------------  symm.c  -----------------------------------*
 *                                                                       *
 * Calculates the symmetries of the system,                              *
 *                                                                       *
 * Fold and reduce the k-points.                                         *
 *                                                                       *
 * And symmetrize the charge density.                                    *
 *                                                                       *
 *-----------------------------------------------------------------------*/
#ifndef DFT_SYMM_H
#define DFT_SYMM_H

void symmetries(Ioninfo *ioninfo, Lattice *lattice, Symmetries *symm);

void bravais_symmetries(Lattice *lattice, int &bnrot, matrix3 *bsym);

void minimize_basis(matrix3 &a, matrix3 &t);

real matcmp(matrix3 a, matrix3 b);

void basis_symmetries(Ioninfo *ioninfo, int &bnrot, matrix3 *bsym, 
		      vector3 tr, int &tnrot, matrix3 *tsym);
void check_symmetries(Ioninfo *ioninfo, Symmetries *symm);

int fold_kpoints(vector3 *old_kvec, vector3 **new_kvec,
		  real *old_w, real **new_w, 
		  const int *kpt_fold, int nkpts, int &new_nkpts);

void reduce_kpoints(const vector3 *old_kvec, real *old_w, 
		    vector3 **new_kvec, real **new_w, 
		    int nkpts,
		    int &new_nkpts,
		    const Symmetries &symm,
		    Elecinfo &elecinfo, const Lattice &lattice,
		    int reduce_kpts_flag);


int check_basis_symm_compatibility(Symmetries &symm, const Basis &basis);

int map_symm_atom(Ioninfo &ioninfo, const Symmetries &symm);

#endif // DFT_SYMM_H
